/****************************************************************************
 ****				  PREPROM.C				 ****
 *--------------------------------------------------------------------------*
 * Modul	:							    *
 * Language	: Borland C 3.1 					    *
 * Destination	: DOS 6.22						    *
 * Author	: Detlef Mueller, 1999/04/06				    *
 * Description	:							    *
 * To do	:							    *
 * Variables	:							    *
 * Functions	:							    *
 *--------------------------------------------------------------------------*
 * Edition History :							    *
 *    0.000 1999/04/06 DM start initial coding				    *
 ****************************************************************************/


/****************************************************************************
 * RCS history
 *
 * $Id: preprom.c 1.1 1999/04/07 22:02:21 Detlef Exp $
 *
 * $Log: preprom.c $
 * Revision 1.1  1999/04/07 22:02:21  Detlef
 * Initial revision
 *
 ****************************************************************************/

/* -->--> */

/****************************************************************************
 *			       I N C L U D E S 				    *
 ****************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>

#include	<io.h>				/* MesS-DOS */
#include	<fcntl.h>			/* MesS-DOS */


/****************************************************************************
 *			        D E F I N E S 				    *
 ****************************************************************************/


#define	_1k		1024

#define	BUFLEN		_1k
#define	ROMSZ		(512 * _1k)

#define	HROMBLKS	256


/* -->--> */

/****************************************************************************
 ****		   F U N C T I O N   D E F I N I T I O N S		 ****
 ****************************************************************************/

/** Error *******************************************************************
 * Interface :								    *
 *	Error( msg ) ;							    *
 *--------------------------------------------------------------------------*
 * Parameter :								    *
 *	char *msg	--> to string to output before quiting		    *
 *--------------------------------------------------------------------------*
 * Description :							    *
 ****************************************************************************/

void Error ( char *msg )
{
    fprintf( stderr, "\n%s\n\n", msg ) ;
    exit( -1 ) ;
}

/** main ********************************************************************
 * Interface :								    *
 *	main								    *
 *--------------------------------------------------------------------------*
 * Description :							    *
 *	Put all the stuff together ...					    *
 ****************************************************************************/

int main ( int argc, char *argv[] )
{
    static const char
	ver[] = "$Id: preprom.c 1.1 1999/04/07 22:02:21 Detlef Exp $" ;
    static char
	buffer[BUFLEN],
	work[] = "/-\\|" ;

    int
	wi = -1 ;

    char
	*nt ;

    FILE
	*ft ;

    size_t
	i,
	blk = HROMBLKS ;

    fprintf( stderr, "\n%s (c) 1999 DM.\n\n ", ver ) ;

    setmode( fileno( stdout ), O_BINARY ) ;	/* MesS-DOS */
    setmode( fileno( stdin ), O_BINARY ) ;	/* MesS-DOS */

    if ( ! (nt = tempnam( NULL, "ROM" )) )
	Error( "Can't create temporary filename" ) ;

    if ( ! (ft = fopen( nt, "wb" )) )
	Error( "Can't create temporary file" ) ;

    /* Start w/ lower half of ROM. Need to patch the reg. region to 0 */

    if ( (i = fread( buffer, sizeof( char ), BUFLEN, stdin )) != BUFLEN )
	Error( "Input too small" ) ;

    for ( i = 128 ; i <= 159 ; ++i )
	buffer[i] = 0 ;

    if ( (i = fwrite( buffer, sizeof( char ), BUFLEN, ft )) != BUFLEN )
	Error( "Can't write to temporary file" ) ;

    for ( ; --blk ; )
    {
	i = fread( buffer, sizeof( char ), BUFLEN, stdin ) ;
	fwrite( buffer, sizeof( char ), i, ft ) ;

	if ( ferror( stdin ) || ferror( ft ) || i != BUFLEN )
	    Error( "Can't copy stdin to temporary file, maybe input too short" ) ;

	fputc( '\b', stderr ) ;
	fputc( work[wi = (wi + 1) & 3], stderr ) ;
    }

    fclose( ft ) ;

    for ( blk = HROMBLKS ; blk-- ; )
    {
	i = fread( buffer, sizeof( char ), BUFLEN, stdin ) ;
	fwrite( buffer, sizeof( char ), i, stdout ) ;

	if ( ferror( stdin ) || ferror( stdout ) || i != BUFLEN )
	    Error( "Can't copy stdin to stdout, maybe input too short" ) ;

	fputc( '\b', stderr ) ;
	fputc( work[wi = (wi + 1) & 3], stderr ) ;
    }

    if ( ! (ft = fopen( nt, "rb" )) )
	Error( "Can't reopen temporary file" ) ;

    for ( blk = HROMBLKS ; blk-- ; )
    {
	i = fread( buffer, sizeof( char ), BUFLEN, ft ) ;
	fwrite( buffer, sizeof( char ), i, stdout ) ;

	if ( ferror( stdin ) || ferror( ft ) || i != BUFLEN )
	    Error( "Can't copy temporary file to stdout" ) ;

	fputc( '\b', stderr ) ;
	fputc( work[wi = (wi + 1) & 3], stderr ) ;
    }

    fclose( ft ) ;
    unlink( nt ) ;
    free( nt ) ;

    fprintf( stderr, "\b \nDone.\n" ) ;

    exit( 0 ) ;
}

/****************************************************************************
 ****			      End of PREPROM.C				 ****
 ****************************************************************************/
